import PopupManager from '@opentiny/vue-renderless/common/deps/popup-manager';
import { addClass, getStyle, removeClass } from '@opentiny/vue-renderless/common/deps/dom';
import { $prefix, $props, $setup, createComponent, hooks, directive as directive$1, setupComponent } from '@opentiny/vue-common';
import PCTemplate from './pc';
import MobileTemplate from './mobile';
import afterLeave from '@opentiny/vue-renderless/common/deps/after-leave';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);
  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }
  return keys;
}
function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }
  return target;
}
function _defineProperty(obj, key, value) {
  key = _toPropertyKey(key);
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}
function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];
  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (hint === "string" ? String : Number)(input);
}
function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");
  return typeof key === "symbol" ? key : String(key);
}

var template = function template() {
  if (process.env.TINY_MODE === 'pc') {
    return PCTemplate;
  } else if (process.env.TINY_MODE === 'mobile') {
    return MobileTemplate;
  } else {
    return tinyMode === 'mobile' ? MobileTemplate : PCTemplate;
  }
};
var Loading = {
  name: $prefix + 'Loading',
  emits: [],
  props: _objectSpread2(_objectSpread2({}, $props), {}, {
    type: {
      type: String,
      validator: function validator(value) {
        return ~['primary', 'simple'].indexOf(value);
      }
    },
    loadtext: {
      type: String,
      default: function _default() {
        return constants.LOAD_ICON_TEXT;
      }
    },
    _constants: Object
  }),
  setup: function setup(props, context) {
    return $setup({
      props: props,
      context: context,
      template: template
    });
  }
};

var defaults = {
  text: null,
  body: false,
  lock: false,
  customClass: '',
  fullscreen: true,
  type: 'primary',
  tiny_mode: 'pc'
};
var fullscreenLoading = null;
var tinyMode = null;
var constants = {
  TEXT_ATTR: 'tiny-loading__text',
  IS_FULLSCREEN_CLS: 'is-fullscreen',
  TEXT_SPINNER: 'tiny-loading__spinner',
  TEXT_BACKGROUND: 'tiny-loading__background',
  TEXT_CUSTOM_CLS: 'tiny-loading__custom-class',
  PARENT_HIDDEN_CLS: 'tiny-loading__parent-hidden',
  PARENT_RELATIVE_CLS: 'tiny-loading__parent-relative',
  LOAD_ICON_TEXT: 'ui.load.dot'
};
var addStyle = function addStyle(options, parent, instance) {
  var maskStyle = {};
  if (options.fullscreen) {
    instance.originalPosition = getStyle(document.body, 'position');
    instance.originalOverflow = getStyle(document.body, 'overflow');
    maskStyle.zIndex = PopupManager.nextZIndex();
  } else if (options.body) {
    var clientRect = options.target.getBoundingClientRect();
    instance.originalPosition = getStyle(document.body, 'position');
    var direction = ['top', 'left'];
    direction.forEach(function (property) {
      var scroll = property === 'top' ? 'scrollTop' : 'scrollLeft';
      maskStyle[property] = clientRect[property] + document.body[scroll] + document.documentElement[scroll] + 'px';
    });
    var size = ['height', 'width'];
    size.forEach(function (property) {
      maskStyle[property] = clientRect[property] + 'px';
    });
  } else {
    instance.originalPosition = getStyle(parent, 'position');
  }
  Object.keys(maskStyle).forEach(function (property) {
    instance.$el.style[property] = maskStyle[property];
  });
};
var service = (function () {
  var configs = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  configs = _objectSpread2(_objectSpread2({}, defaults), configs);
  tinyMode = configs.tiny_mode;
  if (typeof configs.target === 'string') {
    configs.target = document.querySelector(configs.target);
  }
  configs.target = configs.target || document.body;
  if (configs.target !== document.body) {
    configs.fullscreen = false;
  } else {
    configs.body = true;
  }
  if (configs.fullscreen && fullscreenLoading && !fullscreenLoading.state.closed) {
    return fullscreenLoading;
  }
  var parent = configs.body ? document.body : configs.target;
  var instance = createComponent({
    component: Loading,
    propsData: {
      _constants: constants
    },
    el: document.createElement('div')
  });
  for (var key in configs) {
    if (Object.prototype.hasOwnProperty.call(configs, key)) {
      instance.state[key] = configs[key];
    }
  }
  addStyle(configs, parent, instance);
  if (instance.originalPosition !== 'absolute' && instance.originalPosition !== 'fixed') {
    addClass(parent, constants.PARENT_RELATIVE_CLS);
  }
  if (configs.fullscreen && configs.lock) {
    addClass(parent, constants.PARENT_HIDDEN_CLS);
  }
  parent.appendChild(instance.$el);
  hooks.nextTick(function () {
    instance.state.visible = true;
  });
  if (configs.fullscreen) {
    fullscreenLoading = instance;
  }
  return instance;
});

var insertDom = function insertDom(parent, el, binding) {
  if (!el.domVisible && getStyle(el, 'display') !== 'none' && getStyle(el, 'visibility') !== 'hidden') {
    Object.keys(el.maskStyle).forEach(function (property) {
      el.mask.style[property] = el.maskStyle[property];
    });
    if (el.originalPosition !== 'absolute' && el.originalPosition !== 'fixed') {
      addClass(parent, constants.PARENT_RELATIVE_CLS);
    }
    if (binding.modifiers.fullscreen && binding.modifiers.lock) {
      addClass(parent, constants.PARENT_HIDDEN_CLS);
    }
    el.domVisible = true;
    parent.appendChild(el.mask);
    hooks.nextTick(function () {
      if (el.instance.hiding) {
        el.instance.$emit('after-leave');
      } else {
        el.instance.state.visible = true;
      }
    });
    el.domInserted = true;
  } else if (el.domVisible && el.instance.hiding === true) {
    el.instance.state.visible = true;
    el.instance.hiding = false;
  }
};
var appendLoadingToBody = function appendLoadingToBody(el, binding) {
  var clientRect = el.getBoundingClientRect();
  el.originalPosition = getStyle(document.body, 'position');
  var direction = ['top', 'left'];
  direction.forEach(function (property) {
    var scroll = property === 'top' ? 'scrollTop' : 'scrollLeft';
    el.maskStyle[property] = clientRect[property] + document.body[scroll] + document.documentElement[scroll] - parseInt(getStyle(document.body, "margin-".concat(property)), 10) + 'px';
  });
  var size = ['height', 'width'];
  size.forEach(function (property) {
    el.maskStyle[property] = clientRect[property] + 'px';
  });
  insertDom(document.body, el, binding);
};
var toggleLoading = function toggleLoading(el, binding) {
  if (binding.value) {
    hooks.nextTick(function () {
      if (binding.modifiers.fullscreen) {
        el.originalPosition = getStyle(document.body, 'position');
        el.originalOverflow = getStyle(document.body, 'overflow');
        el.maskStyle.zIndex = PopupManager.nextZIndex();
        addClass(el.mask, constants.IS_FULLSCREEN_CLS);
        insertDom(document.body, el, binding);
      } else {
        removeClass(el.mask, constants.IS_FULLSCREEN_CLS);
        if (binding.modifiers.body) {
          appendLoadingToBody(el, binding);
        } else {
          el.originalPosition = getStyle(el, 'position');
          insertDom(el, el, binding);
        }
      }
    });
  } else {
    afterLeave(el.instance, function () {
      if (!el.instance.hiding) {
        return;
      }
      var target = binding.modifiers.fullscreen || binding.modifiers.body ? document.body : el;
      el.domVisible = false;
      removeClass(target, constants.PARENT_RELATIVE_CLS);
      removeClass(target, constants.PARENT_HIDDEN_CLS);
      el.instance.hiding = false;
    }, 300, true);
    el.instance.state.visible = false;
    el.instance.hiding = true;
  }
};
var vLoading = {
  bind: function bind(el, binding, vnode) {
    var vm = vnode.context;
    var textExr = el.getAttribute(constants.TEXT_ATTR);
    var spinnerExr = el.getAttribute(constants.TEXT_SPINNER);
    var backgroundExr = el.getAttribute(constants.TEXT_BACKGROUND);
    var customClassExr = el.getAttribute(constants.TEXT_CUSTOM_CLS);
    var mask = createComponent({
      component: Loading,
      propsData: {
        _constants: constants
      },
      el: document.createElement('div')
    });
    var config = {
      text: vm && vm[textExr] || textExr,
      spinner: vm && vm[spinnerExr] || spinnerExr,
      background: vm && vm[backgroundExr] || backgroundExr,
      customClass: vm && vm[customClassExr] || customClassExr,
      fullscreen: !!binding.modifiers.fullscreen
    };
    for (var key in config) {
      if (Object.prototype.hasOwnProperty.call(config, key)) {
        mask.state[key] = config[key];
      }
    }
    el.instance = mask;
    el.mask = mask.$el;
    el.maskStyle = {};
    binding.value && toggleLoading(el, binding);
  },
  update: function update(el, binding) {
    el.instance.setText(el.getAttribute(constants.TEXT_ATTR));
    if (binding.oldValue !== binding.value) {
      toggleLoading(el, binding);
    }
  },
  unbind: function unbind(el, binding) {
    if (el.domInserted) {
      el.mask && el.mask.parentNode && el.mask.parentNode.removeChild(el.mask);
      toggleLoading(el, {
        value: false,
        modifiers: binding.modifiers
      });
    }
  }
};
var directive = directive$1({
  vLoading: vLoading
}).vLoading;

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".tiny-loading{--ti-loading-text-color:var(--ti-base-color-brand-6);--ti-loading-mask-bg-color:rgba(255, 255, 255, 0.9);--ti-loading-text-font-size:var(--ti-common-font-size-1);--ti-loading-spinner-height:42px;--ti-loading-spinner-width:42px;--ti-loading-fullscreen-height:50px;--ti-loading-fullscreen-width:50px}.tiny-loading__mask{position:absolute;z-index:2000;background-color:var(--ti-loading-mask-bg-color);margin:0;top:0;right:0;bottom:0;left:0;-webkit-transition:opacity .3s;transition:opacity .3s}.tiny-loading__mask.is-fullscreen{position:fixed}.tiny-loading__mask.is-fullscreen .tiny-loading__spinner{margin-top:-25px}.tiny-loading__mask.is-fullscreen .tiny-loading__spinner .circular{height:var(--ti-loading-fullscreen-height);width:var(--ti-loading-fullscreen-width)}.tiny-loading__spinner{top:50%;margin-top:-21px;width:100%;text-align:center;position:absolute}.tiny-loading__spinner .tiny-loading__text{color:var(--ti-loading-text-color);margin:3px 0;font-size:var(--ti-loading-text-font-size)}.tiny-loading__spinner .circular{height:var(--ti-loading-spinner-height);width:var(--ti-loading-spinner-width);-webkit-animation:loading-rotate 2s linear infinite;animation:loading-rotate 2s linear infinite}.tiny-loading__spinner .path{-webkit-animation:loading-dash 1.5s ease-in-out infinite;animation:loading-dash 1.5s ease-in-out infinite;stroke-dasharray:90,150;stroke-dashoffset:0;stroke-width:2;stroke:var(--ti-loading-text-color);stroke-linecap:round}.tiny-loading__spinner svg{fill:var(--ti-loading-text-color)}.tiny-loading__parent-relative{position:relative!important}.tiny-loading__parent-hidden{overflow:hidden!important}.tiny-loading-fade-enter,.tiny-loading-fade-leave-active{opacity:0}@-webkit-keyframes loading-rotate{100%{-webkit-transform:rotate(360deg);transform:rotate(360deg)}}@keyframes loading-rotate{100%{-webkit-transform:rotate(360deg);transform:rotate(360deg)}}@-webkit-keyframes loading-dash{0%{stroke-dasharray:1,200;stroke-dashoffset:0}50%{stroke-dasharray:90,150;stroke-dashoffset:-40px}100%{stroke-dasharray:90,150;stroke-dashoffset:-120px}}@keyframes loading-dash{0%{stroke-dasharray:1,200;stroke-dashoffset:0}50%{stroke-dasharray:90,150;stroke-dashoffset:-40px}100%{stroke-dasharray:90,150;stroke-dashoffset:-120px}}.tiny-icon-loading{font-size:var(--ti-common-font-size-1);line-height:1;vertical-align:text-top;-webkit-animation:rotating 2s linear infinite;animation:rotating 2s linear infinite}@-webkit-keyframes rotating{0%{-webkit-transform:rotateZ(0);transform:rotateZ(0)}100%{-webkit-transform:rotateZ(360deg);transform:rotateZ(360deg)}}@keyframes rotating{0%{-webkit-transform:rotateZ(0);transform:rotateZ(0)}100%{-webkit-transform:rotateZ(360deg);transform:rotateZ(360deg)}}";
styleInject(css_248z);

var Loadings = {};
Loadings.install = function (app) {
  app.directive('loading', directive);
};
setupComponent.TINYLoading = {
  init: function init(root) {
    root.$loading = service;
  }
};
Loadings.service = service;
Loadings.directive = directive;
Loadings.version = '3.6.0-alpha.0';

export { Loadings as default };
